# -----------------------------------------------------------------------------
sub infCleaners
# Note: this subroutine must follow check_dbs
{
  ($thishost) = @_;

  if ($up[$thishost] ne "Y")
  {
    logit("(Down/Inhibited)",$thishost);
    return 1;
  }

  if ($num_disks[$thishost] eq "")
  {
    logit("Disk count null - cannot run cleaners check",$thishost);
    return 0;
  } 

  $cleanpct = int($cleaners[$thishost] / $num_disks[$thishost] * 100);
  $recnumcl = $num_disks[$thishost] + 2;

  if ($recnumcl > 127)
  {
    $recnumcl = 127;
  }

  if ($recnumcl < $lrus[$thishost])
  {
    $recnumcl = $lrus[$thishost];
  }

  ##print "cl=" cleaners[$thishost] 
  ##print "nd=" num_disks[$thishost] 
  ##print "lr=" lrus[$thishost] 
  ##print "pc=" cleanpct
  ##print "rn=" recnumcl

  &logDetails($thishost,"<h3>Informix Page Cleaner Data</h3>");
  &logDetails($thishost,"<table border=1 bgcolor=$table_bgcolor>");
  &logDetails($thishost,"<tr bgcolor=slate><th>Parameter<th>Value<th>Comments</tr>");
  &logDetails($thishost,"<tr><td>$f_s Current CLEANERS ONCONFIG Value:<td>$f_s $cleaners[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s Current LRUS ONCONFIG value:<td>$f_s $lrus[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s Current Number of Disks:<td>$f_s $num_disks[$thishost]");
  &logDetails($thishost,"<tr><td>$f_s % of Disks That Have a Cleaner:<td>$f_s $cleanpct");
  &logDetails($thishost,"<tr><td>$f_s % of Disks ... Threshold:<td>$f_s 75");
  &logDetails($thishost,"</table><hr>");

  if (($cleanpct < 75) || ($cleaners[$thishost] < ($lrus[$thishost] / 2) ))
  { 
    #
    # Added code for big instances (boi90) that have 127 CLEANERS)
    #
    if ($cleaners[$thishost] < 127)
    {
      open(TMPFL6, ">$tmpfl6") || &error_msg("Can't open output $tmpfl6 - $!");
      print TMPFL6 "Current CLEANERS ONCONFIG value: $cleaners[$thishost] \n";
      print TMPFL6 "Current LRUS ONCONFIG value:     $lrus[$thishost]  \n";
      print TMPFL6 "Current number of disks:         $num_disks[$thishost]  \n";
      print TMPFL6 "% of disks that have a CLEANER:  $cleanpct  \n";
      print TMPFL6 "Recommended CLEANERS value:      $recnumcl  \n";
      close(TMPFL6);
      &event("P",$thishost,"CLEANERS ($cleaners[$thishost]) must be at least 75% of the number of disks ($num_disks[$thishost]) and >= (LRUS/2) (LRUS=$lrus[$thishost])","Clnr","DBA330",$tmpfl6);
    }
  }
  else
  {
    $msg = sprintf("OK - CLEANERS=%s disks=%s (%s percent) of disks",$cleaners[$thishost],$num_disks[$thishost],$cleanpct);
    logit($msg,$thishost);
  }

  #
  # Check AIO vp's
  #
  # The var AIORAT is the percentage the last AIOVP totalops is vs. 
  # the first AIOVP. If that is > 10 (meaning that the last is more
  # than 10% as busy as the first) then we need to increase the 
  # number of AIOVP's. The recommendation will be to make it 50% 
  # of cleaners. If this value is not sufficient, then hmmmmm.
  #
  # bb 03Jun99 - changed above 10 number to 40 - if there are too many
  #   aiovps, you can exceed NFILES which causes down chunks.

  $aiong = 0;

  $recaiovp = int($cleaners[$thishost] * 0.75);
  if ($recaiovp <= $numaiovps[$thishost])
  {
    $recaiovp = int($numaiovps[$thishost] * 1.25);
  }
  $toadd = $recaiovp - $numaiovps[$thishost] ;

  #
  # If the recommended number of aiovps is > 2x cleaners, don't
  # bother trying to increase AIOvps.
  #
  if ($recaiovp > ($cleaners[$thishost] * 2))
  {
    $proceed = 0;
  }
  else
  {
    $proceed = 1;
  }

  #
  # If the recommended number of aiovps to add is 0, don't bother
  #
  if ($toadd == 0)
  {
    $proceed = 0;
  }

  #
  # Create events
  # 

  if ($kio[$thishost] == 1 && $numaiovps[$thishost] > 2)
  { 
    $aiong = 1;

    open(TMPFL6, ">$tmpfl6") || &error_msg("Can't open output $tmpfl6 - $!");
    print TMPFL6 "Current NUMAIOVPS ONCONFIG value: $numaiovps[$thishost] \n";
    print TMPFL6 "Current Number of AIO VPs:        $aiocur[$thishost] \n";
    print TMPFL6 "Recommended NUMAIOVPS value:      2\n";
    close(TMPFL6);
    &event("P",$thishost,"(KAIO on) - NUMAIOVPS value of $numaiovps[$thishost] is greater than recommended value of 1 or 2","AIOk","DBA360I",$tmpfl6);
  }

  if ($kio[$thishost] != 1 && $aiorat[$thishost] > 40 && $proceed == 1)
  { 
    $aiong = 1;

    open(TMPFL6, ">$tmpfl6") || &error_msg("Can't open output $tmpfl6 - $!");
    print TMPFL6 "AIOVP Least to Most Busy TOTALOPS Ratio: $aiorat[$thishost] \n";
    print TMPFL6 "Current NUMAIOVPS ONCONFIG value:        $numaiovps[$thishost] \n";
    print TMPFL6 "Current Number of AIO VPs:               $aiocur[$thishost] \n";
    print TMPFL6 "Current CLEANERS ONCONFIG value:         $cleaners[$thishost] \n";
    print TMPFL6 "Recommended NUMAIOVPS value:             $recaiovp \n\n";
    print TMPFL6 "Note: Ensure that you have sufficient NFILE's before adding AIO VP's.\n\n" ;
    print TMPFL6 "Note: this could be caused by NUMAIOVPS being configured\n" ;
    print TMPFL6 "low for KAIO, and the KAIOON environment variable not being\n" ;
    print TMPFL6 "set to 1 at Informix startup time.\n\n" ;
    print TMPFL6 "To solve this problem now, dynamically add $toadd AIO VP's\n" ;
    print TMPFL6 "by running 'onmode -p +$toadd aio'. Also remember to change\n";
    print TMPFL6 "the NUMAIOVPS ONCONFIG value to $recaiovp .\n";
    close(TMPFL6);
    &event("P",$thishost,"(KAIO off) - AIO Least to Most Ratio $aiorat[$thishost] > Threshold of 40 - NUMAIOVPS value: $numaiovps[$thishost] too small","AIOr","DBA370",$tmpfl6);
  }

  ##print "aiong=" aiong

  if ($aiong == 0)
  {
    $msg = sprintf("OK - NUMAIOVPS=%s KIO=%s CLEANERS=%s AIORAT=%s",$numaiovps[$thishost],$kio[$thishost],$cleaners[$thishost],$aiorat[$thishost]);
    &logit($msg,$thishost);
  }
}
# -----------------------------------------------------------------------------
1;
